// PRE-RELASE VERSION SUBJECT TO CHANGE
// Please do not distribute.

// Template Numerical Toolkit (TNT) for Linear Algebra 
// Kernel algorithms for TNT default types "Matrix(T)"
//    and "Vector(T)"
//
// R. Pozo
// Applied and Computational Mathematics Division
// National Institute of Standards and Technology


#ifndef ALGOMAT_H
#define ALGOMAT_H

template <class T>
Matrix(T) outer_product(const Vector(T) &x1, const Vector(T) &x2)
{
    Subscript N = v1.dim();
    assert(N == v2.dim());

    Matrix(T) Tmp(N,N);

    Subscript i,j;

    for (i=1; i<=M; i++)
        for (j=1; j<=N; j++)
            M(i,j) = v1(i)*v2(j);

    return Tmp;
}

template <class T>
Matrix(T) operator+(const Matrix(T) &A, const Matrix(T) &B)
{
    Subscript M = A.num_rows();
    Subscript N = A.num_cols();

    assert(M==B.num_rows());
    assert(N==B.num_cols());

    Matrix(T) tmp(M,N);
    Subscript i,j;

    for (i=1; i<=M; i++)
        for (j=1; j<=N; j++)
            tmp(i,j) = A(i,j) + B(i,j);

    return tmp;
}

template <class T>
Matrix(T) operator-(const Matrix(T) &A, const Matrix(T) &B)
{
    Subscript M = A.num_rows();
    Subscript N = A.num_cols();

    assert(M==B.num_rows());
    assert(N==B.num_cols());

    Matrix(T) tmp(M,N);
    Subscript i,j;

    for (i=1; i<=M; i++)
        for (j=1; j<=N; j++)
            tmp(i,j) = A(i,j) - B(i,j);

    return tmp;
}

// element-wise multiplication  (use matmult() below for matrix
// multiplication in the linear algebra sense.)
//
//
template <class T>
Matrix(T) operator*(const Matrix(T) &A, const Matrix(T) &B)
{
    Subscript M = A.num_rows();
    Subscript N = A.num_cols();

    assert(M==B.num_rows());
    assert(N==B.num_cols());

    Matrix(T) tmp(M,N);
    Subscript i,j;

    for (i=1; i<=M; i++)
        for (j=1; j<=N; j++)
            tmp(i,j) = A(i,j) * B(i,j);

    return tmp;
}




//
// Compute Householder Matrices
//
// Given vector v, compute Householder Matrix: (I - u*u')
//
template <class T>
Matrix(T)  Householder(const Vector(T) &v, const Matrix(T) &M)
{
    Subscript N = v.dim();

    Matrix(T) Tmp(N,N);

    Subscript i,j;

    for (i=1; i<=N; i++)
        for (j=1; j<=N; j++)
            Tmp(i,j) = -1 * v(i)*v(j);

    for (i=1; i<=N; i++)
        Tmp(i,j) = Tmp(i,j) + 1;

    return Tmp;
}

    
template <class T>
inline Matrix(T) matmult(const Matrix(T)  &A, const Matrix(T) &B)
{

#ifdef TNT_BOUNDS_CHECK
    assert(A.num_cols() == B.num_rows());
#endif

    Subscript M = A.num_rows();
    Subscript N = A.num_cols();
    Subscript K = B.num_cols();

    Matrix(T) tmp(M,K);
    T sum;

    for (Subscript i=1; i<=M; i++)
    for (Subscript k=1; k<=K; k++)
    {
        sum = 0;
        for (Subscript j=1; j<=N; j++)
            sum = sum +  A(i,j) * B(j,k);

        tmp(i,k) = sum; 
    }

    return tmp;
}


template <class T>
Vector(T) matmult(const Matrix(T)  &A, const Vector(T) &x)
{

#ifdef TNT_BOUNDS_CHECK
    assert(A.num_cols() == x.dim());
#endif

    Subscript M = A.num_rows();
    Subscript N = A.num_cols();

    Vector(T) tmp(M);
    T sum;

    for (Subscript i=1; i<=M; i++)
    {
        sum = 0;
        for (Subscript j=1; j<=N; j++)
            sum = sum +  A(i,j) * x(j);

        tmp(i) = sum; 
    }

    return tmp;
}


template <class T>
Vector(T) operator+(const Vector(T) &A, const Vector(T) &B)
{
    Subscript N = A.dim();

    assert(M==B.dim());

    Vector(T) tmp(M);
    Subscript i;

    for (i=1; i<=M; i++)
        tmp(i) = A(i) + B(i);

    return tmp;
}

template <class T>
Vector(T) operator-(const Vector(T) &A, const Vector(T) &B)
{
    Subscript N = A.dim();

    assert(N==B.dim());

    Vector(T) tmp(N);
    Subscript i;

    for (i=1; i<=N; i++)
        tmp(i) = A(i) - B(i);

    return tmp;
}



template <class T>
Vector(T) Lower_triangular_solve(const Matrix(T) &A, const Vector(T) &b)
{
    Subscript M = A.num_rows();
    Subscript N = A.num_cols();
    Subscript Nb = b.dim();


                        // n = min(M, N, Nb);

    Subscript n = ( M < N ? M : N);  // n = min(M,N);
              n = (n < Nb ? n : Nb); // n = min(n, Nb);

    if (n<1) return Vector(T)(0);

    Vector x(n);
    Matrix::element_type tmp=0;

    x(1) = b(1) / A(1,1);
    for (Subscript i=2; i<=n; i++)
    {
        tmp = 0;
        for (Subscript j=1; j<i; j++) 
            tmp = tmp + A(i,j)*x(j);
        x(i) =  (b(i) - tmp) / A(i,i);
    }

    return x;
}

template <class T>
Vector(T) Upper_triangular_solve(Matrix(T) &A, Vector(T) &b)
{
    Subscript M = A.num_rows();
    Subscript N = A.num_cols();
    Subscript Nb = b.dim();


                        // n = min(M, N, Nb);

    Subscript n = ( M < N ? M : N);  // n = min(M,N);
              n = (n < Nb ? n : Nb); // n = min(n, Nb);

    Vector x(n);
    Matrix::element_type tmp=0;

    x(n) = b(n) / A(n,n);
    for (Subscript i=n-1; i>=1; i--)
    {
        tmp = 0;
        for (Subscript j=i+1; j<=n; j++) 
            tmp = tmp + A(i,j)*x(j);
        x(i) =  (b(i) - tmp) / A(i,i);
    }

    return x;
}

template <class Matrix>
Matrix Lower_triangular_part(Matrix &A)
{
    Matrix tmp(A);
    Subscript M = A.num_rows();
    Subscript N = A.num_cols();

    Subscript n = (M < N ? M : N);

    return tmp;
}

#endif
// ALGOMAT_H
