// PRE-RELASE VERSION SUBJECT TO CHANGE
// Please do not distribute.

// Template Numerical Toolkit (TNT) for Linear Algebra
//
// R. Pozo
// Applied and Computational Mathematics Division
// National Institute of Standards and Technology

// This is the bare-bones templated vector class.  It is used
// as an internal buidling block, with compilers that have
// trouble using the STL vector<> class.  Note that this
// class is *NOT* an STL object, however it can be
// passed into STL algorithms (as a regular C array would).
//
// This class should be fairly efficient and optimized for
// particular platforms.
//
// For full details of TNT building-block  vector class, see
//  file "tntreqs.h".
//
//  NOTE: this class is *NOT* meant to be derived from,
//  so its methods (particularly indexing) need not be
//  decalred virtual.
//
//
//  Some things it *DOES NOT* need to do are
//
//  o) bounds checking
//  o) array referencing (e.g. reference counting)
//  o) support () indexing
//  o) I/O 
//  o) support a destructive resize(N) function
//
//
//************************************************************************



#ifndef BB_VECTOR_H
#define BB_VECTOR_H


#include <assert.h>

template <class T>
class BB_Vector
{


  typedef int Subscript;

#if !defined(NULL)
#define NULL 0
#endif

  protected:
    T *v_;
    Subscript size_;

  public:

    Subscript size() const { return size_; }

  public:

    T* begin(){ return v_; }
    const T* begin() const{ return v_; }
    T* end(){ return v_ + size_; }
    const T* end() const { return v_ + size_; }

    
    BB_Vector() : v_(NULL), size_(0) {};
    ~BB_Vector() { if (v_ != NULL) delete [] v_; }

    BB_Vector<T> & free() 
    { 
        if (v_ != NULL) 
            delete [] v_;
        size_ = 0;
        return *this;
    }

    BB_Vector<T> & newsize(Subscript N)
    {
        free();
        if (N<=0) return *this;

        v_ = new T[N];
        assert(v_ != NULL);
        size_ = N;
        return *this;
    }

    BB_Vector(const BB_Vector &A) : v_(NULL), size_(A.size_) 
    { 
        if (size_ == 0) return;

        v_ = new T [A.size_];
        assert(v_ != NULL);
        for (Subscript i=0; i < size_; i++)
            v_[i] = A.v_[i];
    } 

    BB_Vector(Subscript N) : v_(NULL), size_(N)
    {
        if (N <= 0)  return;
        v_ = new T [N];
        assert(v_ != NULL);
    }

    BB_Vector(Subscript N, const T& scalar) : v_(NULL), size_(N)
    {
        if (N <= 0) return;

        Subscript i;
        v_ = new T [N];
        assert(v_ != NULL);
        for (i=0; i<N; i++)
            v_[i] = scalar;
    }

    BB_Vector(Subscript N, const T* array) : v_(NULL), size_(N)
    {
        if (N <= 0) return;

        Subscript i;
        v_ = new T [N];
        assert(v_ != NULL);
        for (i=0; i<N; i++)
            v_[i] = array[i];
    }

    BB_Vector<T>& operator=(const BB_Vector<T> &A)
    {
        Subscript i;

        
        if (v_ == A.v_)
            return *this;


        if (v_ != NULL) delete [] v_;       // these 4 lines could be
        v_ = new T[A.size_];                // replaced by 
        assert(v_ != NULL);                 //
        size_ = A.size_ ;                   //  resize(A.size_);


        for (i=0; i<size_; i++)
        {
            v_[i] = A.v_[i];
        }

        return *this;
    }

    T& operator[](Subscript i) 
    { 
#ifdef TNT_BOUNDS_CHECK
        assert(i>=0);
        assert(i<size_);
#endif
        return v_[i]; 
    }
    const T&  operator[](Subscript i) const 
    {
#ifdef TNT_BOUNDS_CHECK
        assert(i>=0);
        assert(i<size_);
#endif
        return v_[i]; 
    }

#undef NULL
};

#endif
// BB_VECTOR_H
