// PRE-RELASE VERSION SUBJECT TO CHANGE
// Please do not distribute.

// Template Numerical Toolkit (TNT) for Linear Algebra
// Dense Matrix Module
// Row-major storage
//
//
// R. Pozo
// Applied and Computational Mathematics Division
// National Institute of Standards and Technology

#ifndef ROWMAT_H
#define ROWMAT_H

#include <stdlib.h>
#include <assert.h>
#include <iostream.h>
#include <strstream.h>
#ifdef TNT_USE_REGIONS
#include "region2d.h"
#endif


// see tntreq.h for requirement specifications for underlying
// vector class.
//
// ValVec relies only on 0-based offsets via []
// PointerVec is a 0-based vector of (T*)
// 
//          vector<T>      vector<T*>       T
//
template <class ValVec, class PointerVec, class T>
class Row_matrix 
{


  public:

    typedef Subscript   size_type;
    typedef         T   value_type;
    typedef         T   element_type;
    typedef         T*  pointer;
    typedef         T*  iterator;
    typedef         T&  reference;
    typedef const   T*  const_iterator;
    typedef const   T&  const_reference;

 
  protected:
    ValVec v_;
    Subscript m_;
    Subscript n_;
    PointerVec rows_;

  public:

    // methods that provide matrix information
    //

    iterator begin() { return v_.begin();}
    iterator end()   { return v_.end(); }
    const_iterator begin() const { return v_.begin(); }
    const_iterator end() const { return v_.end(); } 



    Subscript size() const { return v_.size(); }    
    Subscript size(Subscript d) const { 
        return (d==TNT_BASE_OFFSET) ? m_ : ((d==TNT_BASE_OFFSET+1) ? n_ : 0); }
    Subscript dim(Subscript d) const {
        return (d==TNT_BASE_OFFSET) ? m_ : ((d==TNT_BASE_OFFSET+1) ? n_ : 0); }
    Subscript num_rows() const { return m_; }
    Subscript num_cols() const { return n_; }


  protected : 

    // internal helper function to create the array
    // of row pointers

    void initialize_row_pointer(PointerVec &c, Subscript M,
            Subscript N, ValVec &v)
    {
        iterator p = v.begin();
        for (Subscript i=0; i<M; i++)
        {
            c[i] = p;
            p += N ;
        }
    }
    

  public:

    // constructors

    Row_matrix() : v_(), m_(0), n_(0), rows_() {};
    Row_matrix(const Row_matrix &A);
    Row_matrix(const Row_matrix &A, 
        Subscript Mbegin, Subscript Mend, Subscript Nbegin, Subscript Nend);
    Row_matrix(Subscript M, Subscript N, const T& value = T(0));
    Row_matrix(Subscript M, Subscript N, const T* v);
    Row_matrix(Subscript M, Subscript N, /*const*/ char* s);

    // assignments
    //
    Row_matrix<ValVec, PointerVec, T>& operator=(const Row_matrix<ValVec, PointerVec, T> &A);
    Row_matrix<ValVec, PointerVec, T>& operator=(const T& scalar);


    // 1-based element access
    //
    inline reference operator()(Subscript i, Subscript j)
    { 
#ifdef TNT_BOUNDS_CHECK
        assert(TNT_BASE_OFFSET<=i);
        assert(i <= m_ + (TNT_BASE_OFFSET -1));
        assert(TNT_BASE_OFFSET<=j);
        assert(j <= n_ + (TNT_BASE_OFFSET -1));
#endif
        return rows_[i-TNT_BASE_OFFSET][j-TNT_BASE_OFFSET]; 
    }

    inline const_reference operator() (Subscript i, Subscript j) const
    {
#ifdef TNT_BOUNDS_CHECK
        assert(TNT_BASE_OFFSET<=i);
        assert(i <= m_ + (TNT_BASE_OFFSET -1));
        assert(TNT_BASE_OFFSET<=j);
        assert(j <= n_ + (TNT_BASE_OFFSET -1));
#endif
         return rows_[i-TNT_BASE_OFFSET][j-TNT_BASE_OFFSET]; 
    }


#ifdef TNT_USE_REGIONS
  // These are the "index-aware" features


    typedef Region2D<Row_matrix<ValVec, PointerVec, T>,T> Region;
    typedef const_Region2D<Row_matrix<ValVec, PointerVec, T>,T> const_Region;

    Region operator()(const Index1D &I,
            const Index1D &J)
    {
        return Region(*this, I, J);
    }

    Region operator()(Subscript i1,
            Subscript i2, Subscript j1, Subscript j2)
    {
        return Region(*this, i1, i2, j1, j2);
    }

    const_Region operator()(const Index1D &I,
            const Index1D &J) const
    {
        return const_Region(*this, I, J);
    }

    const_Region operator()(Subscript i1, Subscript i2,
            Subscript j1, Subscript j2) const
    {
        return const_Region(*this, i1, i2, j1, j2);
    }
#endif
// TNT_USE_REGIONS

};


template <class ValVec, class PointerVec, class T>
Row_matrix<ValVec, PointerVec, T>::Row_matrix(Subscript M, 
            Subscript N, const T* v): v_(M*N, v), m_(M), n_(N), rows_(M)
{
    initialize_row_pointer(rows_, M, N, v_);
}


template <class ValVec, class PointerVec, class T>
Row_matrix<ValVec, PointerVec, T>
::Row_matrix(Subscript M, Subscript N, 
        const T& value) : v_(M*N, value), m_(M), n_(N), rows_(M)
{
    initialize_row_pointer(rows_, M, N, v_);
}

template <class ValVec, class PointerVec, class T>
Row_matrix<ValVec, PointerVec, T>
::Row_matrix(Subscript M, Subscript N, 
        /*const*/ char *s) : v_(M*N), m_(M), n_(N), rows_(M)
{
    initialize_row_pointer(rows_, M, N, v_);
    istrstream ins(s);

    Subscript i,j;
    Subscript M1 = TNT_BASE_OFFSET -1 + M;
    Subscript N1 = TNT_BASE_OFFSET -1 + N;

    for (i=TNT_BASE_OFFSET; i<= M1; i++)
        for (j=TNT_BASE_OFFSET; j<= N1; j++)
             ins >> (*this)(i,j);
}


template <class ValVec, class PointerVec, class T>
Row_matrix<ValVec, PointerVec, T>::Row_matrix(
    const Row_matrix<ValVec, PointerVec, T> &A) : 
    v_(A.v_), m_(A.m_), n_(A.n_), rows_(A.m_)
{
    initialize_row_pointer(rows_, m_, n_, v_);
}

#if 0
template <class T>
Row_matrix<ValVec, PointerVec, T>::Row_matrix(const Row_matrix<ValVec, PointerVec, T> &A,
        Subscript Mbegin, Subscript Mend, Subscript Nbegin, Subscript Nend):
    v_((Mend-Mbegin+1) * (Nend-Nbegin+1) ), m_(Mend-Mbegin+1), 
        n_(Nend-Nbegin+1), rows_(Mend-Mbegin+1)
{
    Subscript M = (Mend - Mbegin + 1);
    Subscript N = (Nend - Nbegin +1);


    initialize_row_pointer(rows_, M, N, v_);

    M += TNT_BASE_OFFSET;
    N += TNT_BASE_OFFSET;
    for (Subscript i=TNT_BASE_OFFSET; i<M; i++)
        for (Subscript j=TNT_BASE_OFFSET; j<N; j++)
            (*this)(i,j) = A(Mbegin+i, Nbegin+j);

}
#endif


template <class ValVec, class PointerVec, class T>
Row_matrix<ValVec, PointerVec, T>& Row_matrix<ValVec, PointerVec, T>::operator=(
    const Row_matrix<ValVec, PointerVec, T> &A)
{

    v_ = A.v_;

    m_ = A.m_;

    n_ = A.n_;

    rows_ = A.rows_;        // rows_ values are overwritten -- just
                            // want to get the size information right

    initialize_row_pointer(rows_, m_, n_, v_);

    return *this;
}

template <class ValVec, class PointerVec, class T>
Row_matrix<ValVec, PointerVec, T>& Row_matrix<ValVec, PointerVec, T>::operator=( const T& scalar)
{
//  for (iterator p=begin(); p != end(); p++)
//      *p = scalar;

    Subscript N = dim(1)*dim(2);
    for (Subscript i=0; i< N; i++)
        v_[i] = scalar;

    return *this;
}



/* ***************************  I/O  ********************************/
#include <iostream.h>

template <class ValVec, class PointerVec, class T>
ostream& operator<<(ostream &s, const Row_matrix<ValVec, PointerVec, T> &A)
{
    Subscript M=A.num_rows();
    Subscript N=A.num_cols();

    s << M << " " << N << endl;

    for (Subscript i=1; i<=M; i++)
    {
        for (Subscript j=1; j<=N; j++)
        {
            s << A(i,j) << " ";
        }
        s << endl;
    }


    return s;
}

template <class ValVec, class PointerVec, class T>
istream& operator>>(istream &s, Row_matrix<ValVec, PointerVec, T> &A)
{

    Subscript M, N;

    s >> M >> N;


    Row_matrix<ValVec, PointerVec, T> tmp(M,N);


    for (Subscript i=1; i<=M; i++)
        for (Subscript j=1; j<=N; j++)
        {
            s >>  tmp(i,j);
        }

    A = tmp;

    return s;
}




#endif

// ROWMAT_H
