// PRE-RELASE VERSION SUBJECT TO CHANGE
// Please do not distribute.

// Template Numerical Toolkit (TNT) for Linear Algebra
// Header file for TNT default definitions
//
// R. Pozo  (http://math.nist.gov/tnt)
// Applied and Computational Mathematics Division
// National Institute of Standards and Technology

#ifndef TNT_H
#define TNT_H

//---------------------------------------------------------------------
// This definition describes the default TNT data type used for
// indexing into TNT matrices and vectors.  The data type should
// be wide enough to index into large arrays.
//
// Typically, this can something like
// 
//   #include <malloc.h>
//   typedef size_t Subscript;
//
// or
//
//   typedef int Subscript;
//
//---------------------------------------------------------------------
//

typedef int Subscript;

//---------------------------------------------------------------------
// Define this macro if you want  TNT to ensure all refernces
// are within the bounds of the array.  This encurs a run-time
// overhead, of course, but is recommended while developing
// code.  It can be turned off for production runs.
// 
//       #define TNT_BOUNDS_CHECK
//---------------------------------------------------------------------
//
#define TNT_BOUNDS_CHECK

//---------------------------------------------------------------------
// Define this macro if you want to utilize matrix and vector
// regions.  This is typically on, but you can save some
// compilation time by turning it off.  If you do this and
// attempt to use regions you will get an error message.
//
//       #define TNT_USE_REGIONS
//---------------------------------------------------------------------
//
//#define TNT_USE_REGIONS

//---------------------------------------------------------------------
//                  TNT INTERNAL VECTOR CLASS
//
// The TNT matrix and vector class are actually container adaptors.
// They can use any vector class (e.g. STL) that provides minimal
// interface. (See "tntreqs.h" for details.)
//
// By the default, TNT uses its own BB_Vector<> 'bare-bones' implementation.
// used as building blocks to higher-level numerical matrices and vectors.
// This is done mainly for portability, so that the package can be used
// with compilers that cannot process STL.
//
// The default is BB_Vector<> class in "bbvector.h"
//
//      #define TNT_INTERNAL_VECTOR  BB_Vector
//      #include "bbvector.h"
//
//  alternately, to use the STL vector<> class replace this with
//
//      #define TNT_INTERNAL_VECTOR vector
//      #include <vector.h>
//
//  or your own optimized FooVec<> class
//
//      #define TNT_INTERNAL_VECTOR FooVec
//      #include "foo.h"
//
//---------------------------------------------------------------------
//
#define TNT_INTERNAL_VECTOR BB_Vector
#include "bbvector.h"


//---------------------------------------------------------------------
// This is the internal base offset used with () indexing.
// TNT classes such as Vector<> and Row_matrix<> use, by default, 
// 1-based indeces similar to Fortran.
//
// Although this could be changed to any offset, realize that
// many external libraries may no longer work, i.e. don'tchange 
// this unless you know what you are doing.
//---------------------------------------------------------------------
//
#define TNT_BASE_OFFSET (1)


//---------------------------------------------------------------------
//  Use the numerical TNT Vector<> and row-major Row_matrix<> class
//  as the default classes for vector and matrices.
//
//  NOTE: TNT_BASE_OFFSET must be defined before this point.
//---------------------------------------------------------------------
//
#include "vvector.h"
#include "rowmat.h"

//---------------------------------------------------------------------
//  Now creat simple syntax for declaring default classes.
//---------------------------------------------------------------------
//
//
#define Vector(T)       Vector_<TNT_INTERNAL_VECTOR<T>, T>
#define Row_Matrix(T)   Row_matrix<TNT_INTERNAL_VECTOR<T>, TNT_INTERNAL_VECTOR<T*>, T>
#define Matrix(T)       Row_Matrix(T)

#define Vector_double    Vector(double)
#define Vector_float     Vector(float)
#define Vector_int       Vector(int)
#define Vector_Subscript Vector(Subscript)

#define Matrix_double       Matrix(double)
#define Matrix_float        Matrix(float)
#define Matrix_int          Matrix(int)
#define Matrix_Subscript    Matrix(Subscript)


//---------------------------------------------------------------------
// Include basic scalar functions like abs(), max(), sign().
// Most of these should be available on your system, otherwise
// you find them in tntmath.cc.   By default, these are not included
// because they cause conflicts with ANSI platforms.  (For example,
// STL defines it own min() and max() functions.)  
//---------------------------------------------------------------------
//
// 
// #include "tntmath.h"


#endif
// TNT_H




