// PRE-RELASE VERSION SUBJECT TO CHANGE
// Please do not distribute.

// Template Numerical Toolkit (TNT) for Linear Algebra
//
//
// R. Pozo
// Applied and Computational Mathematics Division
// National Institute of Standards and Technology


#ifndef VVECTOR_H
#define VVECTOR_H

#include <stdlib.h>
#include <iostream.h>
#include <strstream.h>
#include <assert.h>

#ifdef TNT_USE_REGIONS
#include "region1d.h"
#endif

//  see "tntreq.h" for TNT requirements for underlying vector
//  class.  This need NOT be the STL vector<> class, but a subset
//  that provides minimal services.
//
//  This is a container adaptor that provides the following services.
//
//      o)  adds operator() access
//      o)  adds TNT_BOUNDS_CHECK to () and []
//      o)  adds initialization from strings, e.g.  "1.0 2.0 3.0";
//      o)  adds resize(N) function  (preserves previous values)
//      o)  adds newsize(N) function (does not preserve previous values)
//      o)  adds dim() and dim(0)
//      o)  adds free() function to release memory used by vector
//      o)  adds regions, e.g. A(Index(1,10)) = ... 


template <class BBVec, class T>
class Vector_
{

  public:
    typedef Subscript   size_type;
    typedef         T   value_type;
    typedef         T*  pointer;
    typedef         T*  iterator;
    typedef         T&  reference;
    typedef const   T*  const_iterator;
    typedef const   T&  const_reference;

  protected:
    BBVec v_;

  public:

    Subscript size() const { return v_.size(); }

    iterator begin() { return &v_[0];}
    iterator end()   { return &v_[0] + v_.size();}
    const_iterator begin() const { return &v_[0]; }
    const_iterator end() const { return &v_[0] + v_.size(); }

    Subscript dim() const { return v_.size(); }
    Subscript dim(Subscript i)
    {
#ifdef TNT_BOUNDS_CHECK
        assert(i==TNT_BASE_OFFSET);
#endif
        return (i==TNT_BASE_OFFSET ? v_.size() : 0 );
    }
    Vector_() : v_() {};
    Vector_(const Vector_ &A) : v_(A.v_) {} ;
    Vector_(Subscript N, /*const*/ char *s) : v_(N) 
    {
        istrstream ins(s);
        for (Subscript i=0; i<N; i++)
            ins >> v_[i] ;
    }; 
    Vector_(Subscript N, const T& value = T()) : v_(N, value) {}; 
    Vector_(Subscript N, const T* values) : v_(N, values) {}; 
    Vector_(const BBVec & A) : v_(A) {};

    // NOTE: this assumes that BBVec(0) constructor creates an 
    //  null vector that does not take up space...  It would be
    //  great to require that BBVec have a corresponding free()
    //  function, but in particular STL vectors do not.
    //
    Vector_<BBVec, T>& free()
    {
        return *this = Vector_<BBVec, T>(0);
    }

    Vector_<BBVec, T>& operator=(const Vector_<BBVec, T> &A) { v_ = A.v_ ; 
                    return *this;}

    Vector_<BBVec, T>& newsize(Subscript N)
    {
        // NOTE: this is not as efficient as it could be
        // but to retain compatiblity with STL interface
        // we cannot assume underlying implementation
        // has a newsize() function.

        return *this = Vector_<BBVec, T>(N);

    }

    Vector_<BBVec, T>& operator=(const T &a) 
    {
        Subscript i;
        Subscript N = v_.size();    
        for (i=0; i<N; i++)
            v_[i] = a;

        return *this;
    }

    Vector_<BBVec, T>& resize(Subscript N) 
    { 
        if (N == size()) return *this;

        Vector_<BBVec, T> tmp(N);
        Subscript n =  (N < size() ? N : size());  // min(N, size());
        Subscript i;

        for (i=0; i<n; i++)
            tmp[i] = v_[i];
            

        return (*this = tmp);

    }


    reference operator()(Subscript i)
    { 
#ifdef TNT_BOUNDS_CHECK
        assert(TNT_BASE_OFFSET<=i);
        assert(i<=dim() +(TNT_BASE_OFFSET-1));
#endif
        return v_[i-TNT_BASE_OFFSET]; 
    }

    const_reference operator() (Subscript i) const
    {
#ifdef TNT_BOUNDS_CHECK
        assert(TNT_BASE_OFFSET<=i);
        assert(i<=dim() +(TNT_BASE_OFFSET-1));
#endif
         return v_[i-TNT_BASE_OFFSET];
    }

    reference operator[](Subscript i)
    { 
#ifdef TNT_BOUNDS_CHECK
        assert(TNT_BASE_OFFSET<=i);
        assert(i<=dim() +(TNT_BASE_OFFSET-1));
#endif
        return v_[i]; 
    }

    const_reference operator[] (Subscript i) const
    {
#ifdef TNT_BOUNDS_CHECK
        assert(TNT_BASE_OFFSET<=i);
        assert(i<=dim() +(TNT_BASE_OFFSET-1));
#endif
         return v_[i];
    }

#ifdef TNT_USE_REGIONS
    // "index-aware" features, all of these are 1-based offsets

    typedef Region1D<Vector_<BBVec, T>,T> Region;

    typedef const_Region1D<Vector_<BBVec, T>,T> const_Region;

    Region operator()(const Index1D &I)
    {   return Region(*this, I); }

    Region operator()(const Subscript i1, Subscript i2)
    {   return Region(*this, i1, i2); }

    const_Region operator()(const Index1D &I) const
    {   return const_Region(*this, I); }

    const_Region operator()(const Subscript i1, Subscript i2) const
    {   return const_Region(*this, i1, i2); }
#endif
// TNT_USE_REGIONS


};

#include <iostream.h>

template <class BBVec, class T>
ostream& operator<<(ostream &s, const Vector_<BBVec, T> &A)
{
    Subscript M=A.size();

    s << M << endl;
    for (Subscript i=1; i<=M; i++)
            s << A(i) << endl;
    return s;
}

template <class BBVec, class T>
istream& operator>>(istream &s, const Vector_<BBVec, T> &A)
{
    Subscript N;
    
    s >> N;

    Vector_<BBVec,T> tmp(N);
    for (Subscript i=1; i<=N; i++)
        s >> tmp(i);

    return s;
}


#endif
